;*****************************************************************************
;
;                   B O O T L O A D E R   F L A S H 
;
;*****************************************************************************

include "m8c.inc"       ; part specific constants and macros
include "memory.inc"


;------------------------------------------------------------------------------
;  Declare Flash RAM storage at top of RAM
;     This data area is reserved for exclusive use by Supervisory operations.
;------------------------------------------------------------------------------

bSSC_KEY1:                equ      F8h   ;F8h  KEY1 and Return code for SSC
bTEMP_SPACE:              equ      F8h   ; temporary storage when not KEY1
bSSC_RETURNCODE:          equ      F8h   ;F8h  result code
bSSC_KEY2:                equ      F9h   ;F9h  supervisory stack ptr key
bSSC_BLOCKID:             equ      FAh   ;FAh  block ID
pSSC_POINTER:             equ      FBh   ;FBh  pointer to data buffer
bTEMP_M:                  equ      FBh   ;FBh  M of Mx+B
bSSC_CLOCK:               equ      FCh   ;FCh  Clock
bTEMP_B:                  equ      FCh   ;FCh  B of Mx+B
bTEMP_Mult:               equ      FDh   ;FDh  ClockW ClockE multiplier
bSSC_DELAY:               equ      FEh   ;FEh  flash macro delay count
bSSC_TEMP_Revision:       equ      FFh 
bSSC_WRITE_ResultCode:    equ      FFh   ;FFh  temporary result code
   
TempE:                    equ      FBh
MultHB:                   equ      FCh
MultLB:                   equ      FDh
ResultHB:                 equ      FEh
ResultLB:                 equ      FFh
  

;------------------------------------------------------------------------------
; SSC_Action macro command codes
;------------------------------------------------------------------------------

FLASH_OPER_KEY:           equ      3Ah   ; flash operation key
FLASH_READ:               equ      1     ; flash read supervisory command
FLASH_WRITE:              equ      2     ; flash write supervisory command
FLASH_ERASE:              equ      3     ; flash erase supervisory command
FLASH_TABLE_READ:         equ      6     ; flash read table command
TEMP_TABLE:               equ      3     ; 

;SSC Return Codes
bSSC_SUCCESS:                    equ      00h   ; success return code


;------------------------------------------------------------------------------
; Supervisory Operation Templates:
;   Each system supervisory call uses the reserved data area a little
;   different. Create overlay templates to improve readability and maintenance.
;------------------------------------------------------------------------------

;Flash Read Operation Template
pSSC_READ_FlashBuffer:           equ      FBh   ; pointer to data buffer
bSSC_READ_Counter:               equ      FDh   ; byte counter


;------------------------------------------------------------------------------
; Programming constants
;------------------------------------------------------------------------------

;Flash Sequence Time Delay
ENCORE_PULSE_WIDTH_PROGRAM:      equ      55h   ; enCoRe II Flash pgmming PW
ENCORE_PULSE_WIDTH_ERASE:        equ      15h   ; enCoRe II Flash erase PW
bSSC_DELAY_VALUE:                equ      56h   ; 100us delay at 12MHz

; Temperature Table Template - returned data after performing a
VALID_TABLE_REVISION:            equ      1     ; Temp Table revision number


area Bootloader(rom, rel)

;------------------------------------------------------------------------------
;  MACRO NAME: SpecialSCC
;
;  DESCRIPTION:
;     Performs locally defined supervisory operations. Provides synchronization
;     with PSoC ICE. If the sequence in the macro is changed, the ICE will not 
;     successfully exectute the supervisor functions.
;     !!! DO NOT CHANGE THIS CODE !!!
;
;  RETURNS:
;     BYTE - Nothing
;  SIDE EFFECTS:
;     A and X registers are destroyed
;
;  PROCEDURE:
;     1) specify a 3 byte stack frame.  Save in [KEY2]
;     2) insert the flash Supervisory key in [KEY1]
;     3) store flash operation function code in A
;     4) call the supervisory code
;------------------------------------------------------------------------------

macro SpecialSSC:
   mov   X, SP                         ; copy SP into X
   mov   A, X                          ; A temp stored in X
   add   A, 3                          ; create 3 byte stack frame
   mov   [bSSC_KEY2], A                ; save stack frame for supervisory code
   mov   [bSSC_KEY1], FLASH_OPER_KEY   ; load the supervisory code
   mov   A,@0                          ; load A with specific Flash operation
   SSC                                 ; SSC call the supervisory code
endm


;------------------------------------------------------------------------------
;  FUNCTION NAME: FlashWriteLite
;
;  DESCRIPTION:
;     Writes 64 bytes of data to the flash at the specified blockId.
;
;     Regardless of the size of the buffer, this routine always writes 64 bytes
;     of data. If the buffer is less than 64 bytes, then the next 64-N bytes of
;     RAM will be written to fill the rest of flash block data.
;
;------------------------------------------------------------------------------
;
;  ASSUMPTIONS:
;     Single block Write
;     Temperature is hardcoded to be 32 degees Celcius
;
;  ARGUMENTS:
;    [SP-4] - Block Number to be written
;    [SP-3] - source pointer
;
;  RETURNS:
;     0 if Successful
;     1 if Invalid Revision Error
;     2 if erase failure
;     3 if Write failure
;
;  PROCEDURE:
;    1 ) Compute the programming pulsewidths
;    2 ) Erase the specified block
;    3 ) Program the specified block
;------------------------------------------------------------------------------

   halt       ; Wandering Code Protection
   halt       ; only way to get to next instruction is jmp and call
   halt       ; three byte is biggest instruction size

FlashWriteLite::
; 1 ) Force clock to 12MHz
   mov   A, reg[CPUCLKCR]      ; Get the CPU Clock Configuration Register
   push  A                     ;     and save it on the stack
   and   A, ~CPUCLK_SEL_EXT    ; Clear the external clock 
   mov   reg[CPUCLKCR], A

   M8C_SetBank1
   mov   A, reg[OSC_CR0]       ; Get the System Oscillator control register
   push  A                     ;     and save it on the stack
   and   A, ~OSC_CR0_CPU       ; Clear the CPU clock selects
   or    A, OSC_CR0_CPU_12MHz  ; Set CPU clock to 12 MHz
   mov   reg[OSC_CR0], A
   M8C_SetBank0

; 2 ) Erase the specified block
   mov  X, SP
   mov  A, [X-6]
   mov  [bSSC_BLOCKID], A              ; Block ID
   and  [bSSC_BLOCKID], 0x7F
   mov  [bSSC_CLOCK], ENCORE_PULSE_WIDTH_ERASE
   mov  [bSSC_DELAY], bSSC_DELAY_VALUE

   SpecialSSC FLASH_ERASE
   
   cmp  [bSSC_RETURNCODE], bSSC_SUCCESS
   jz   Erase_Success
   mov  A,2

   ; Return with Unsuccessful Erase Error
   jmp  EndFlashWrite

Erase_Success:
; 3 ) Program the specified block
   mov  X, SP
   mov  A, [X-6]                       ; Block Number
   mov  [bSSC_BLOCKID], A              ; Block ID
   and  [bSSC_BLOCKID], 0x7F
   mov  A, [X-5]                       ; RAM Pointer
   mov  [pSSC_POINTER], A
   mov  [bSSC_CLOCK], ENCORE_PULSE_WIDTH_PROGRAM
   mov  [bSSC_DELAY], bSSC_DELAY_VALUE

   SpecialSSC FLASH_WRITE

   cmp  [bSSC_RETURNCODE], bSSC_SUCCESS
   jz   Write_Success
   ; Return with Unsuccessful Write Error
   mov  A,3
   jmp  EndFlashWrite

Write_Success:
   ; Return with Success
   mov  A,0

EndFlashWrite:
   swap A, X                              ; save return code

   ; Restore Clock
   pop   A
   M8C_SetBank1
   mov   reg[OSC_CR0], A                  ; Restore org CPU rate
   M8C_SetBank0                           ; Switch back to Bank 0
   pop   A
   mov   reg[CPUCLKCR], A                 ; Restore org CPU rate

   swap A, X                              ; get return code
   ret

;-----------------------------------------------------------------------------
; End of file
;-----------------------------------------------------------------------------


