;*****************************************************************************
;
;                   U S B   B O O T L O A D E R
;
;*****************************************************************************

include "m8c.inc"       ; part specific constants and macros
include "bl_loader.inc"
include "USB.inc"
include "usb_macros.inc"


;-----------------------------------------------------------------------------
; Begin RAM usage.  

; The RAM is overlayed with the application RAM. It should not interfere 
; with the the USB variables located in usbRAM.  See custom.lkp for 
; location of this area.  It should start at __usbRAM_end

area Bootloader_RAM (ram, abs, ovr)

; Buffer in RAM to store received data
bootRamBuffer:         BLK 64

; Buffer that stores the 64 byte Flash data
bootFlashBuffer:       BLK 64

; Next three are modified at ISR context
//GbResetISR:            BLK  1 - USBCV test modification for pass condition
bEp1Count:             BLK  1
bEp2Count:             BLK  1

bootTmp:               BLK  1 ; temp variable
bootByteCount:         BLK  1 ; byte counter
bootRecordChecksum:    BLK  1 ; checksum for rx record
bootSourcePointer:     BLK  1 ; RAM source pointer
bootDestPointer:       BLK  1 ; RAM destination pointer
bootCommand:           BLK  1 ; received command
bootTimeoutDelay:      BLK  1 ; timeout count for bootloader mode
bootErrorCode:         BLK  1 ; error state reported back to PC
    ; 7 6 5 4 3 2 1 0 
    ; | | | | | | | |__ Boot Completed Successfully
    ; | | | | | | |____ Image Verify Error
    ; | | | | | |______ Flash Checksum  Error
    ; | | | | |________ Flash Protection Error
    ; | | | |__________ Command / Packet Checksum Error
    ; | | |____________ Bootload Mode entered
    ; | |______________ Invalid Bootload Key received
    ; |________________ Invalid Command received

bootFlashChecksum:     BLK  2 ; checksum accumulation for user image
bootRomAddress:        BLK  2 ; address of Flash memory
bootBlockCount:        BLK  2 ; block counter
bootBlockId:           BLK  2 ; current Flash block being operated on

; Start of stack for bootloader MUST COME LAST
bootStackStart:        EQU  . ; start bootloader stack after variables

; end of overlay RAM usage
;-----------------------------------------------------------------------------


area usbRAM(ram, rel)
BootSignature::        BLK   2 ; Keep signature around since shared with App


area Bootloader(rom, rel)

;-----------------------------------------------------------------------------
;  FUNCTION NAME:    bootResetIsr
;
;  DESCRIPTION:      
;    ISR to handle USB reset for bootloader
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:        None                  
;
;  RETURNS:          None
;
;  SIDE EFFECTS:     Sets global indicating a USB reset has been detected
;
;-----------------------------------------------------------------------------

bootResetIsr::
    ; Check for bootloader signature
    cmp  [BootSignature], SIGNATURE0
    jnz  UserResetIsr
    cmp  [BootSignature+1], SIGNATURE1
    jnz  UserResetIsr
   // inc  [GbResetISR]  ; signal USB reset event
    ljmp _USB_RESET_ISR

UserResetIsr:
    ; If no match, then redirect to user code's USB_RESET vector
    ljmp userUsbReset


;-----------------------------------------------------------------------------
;  FUNCTION NAME:    bootEp0Isr
;
;  DESCRIPTION:      
;    ISR to handle endpoint0 events for bootloader
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:        None                  
;
;  RETURNS:          None
;
;  SIDE EFFECTS:     Make sure we don't execute user code
;
;-----------------------------------------------------------------------------

bootEp0Isr::
    ; Check for bootloader signature
    cmp  [BootSignature], SIGNATURE0
    jnz  UserEp0Isr
    cmp  [BootSignature+1], SIGNATURE1
    jnz  UserEp0Isr
    ljmp _USB_EP0_ISR

UserEp0Isr:
    ; If no match, then redirect to user code's USB_EP0 vector
    ljmp userUsbEp0


;-----------------------------------------------------------------------------
;  FUNCTION NAME:    bootEp1Isr
;
;  DESCRIPTION:      
;    ISR to handle endpoint1 events for bootloader
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:        None                  
;
;  RETURNS:          None
;
;  SIDE EFFECTS:     Sets global indicating a ep1 event has been detected
;
;-----------------------------------------------------------------------------

bootEp1Isr::
    ; Check for bootloader signature
    cmp  [BootSignature], SIGNATURE0
    jnz  UserEp1Isr
    cmp  [BootSignature+1], SIGNATURE1
    jnz  UserEp1Isr
    inc  [bEp1Count]
    ljmp _USB_EP1_ISR

UserEp1Isr:
    ; If no match, then redirect to user code's USB_EP1 vector
    ljmp userUsbEp1


;-----------------------------------------------------------------------------
;  FUNCTION NAME:    bootEp2Isr
;
;  DESCRIPTION:      
;    ISR to handle endpoint2 events for bootloader
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:        None                  
;
;  RETURNS:          None
;
;  SIDE EFFECTS:     Sets global indicating a ep2 event has been detected
;
;-----------------------------------------------------------------------------

bootEp2Isr::
    ; Check for bootloader signature
    cmp  [BootSignature], SIGNATURE0
    jnz  UserEp2Isr
    cmp  [BootSignature+1], SIGNATURE1
    jnz  UserEp2Isr
    inc  [bEp2Count]
    ljmp _USB_EP2_ISR

UserEp2Isr:
    ; If no match, then redirect to user code's USB_EP2 vector
    ljmp userUsbEp2


;-----------------------------------------------------------------------------
;  FUNCTION NAME:    bootUsbActivity
;
;  DESCRIPTION:      
;    ISR to handle USB activity interrupt
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:        None                  
;
;  RETURNS:          None
;
;  SIDE EFFECTS:     None
;
;-----------------------------------------------------------------------------

bootUsbActivity::
    ; Check for bootloader signature
    cmp  [BootSignature], SIGNATURE0
    jnz  UserActivityIsr
    cmp  [BootSignature+1], SIGNATURE1
    jnz  UserActivityIsr
    reti

UserActivityIsr:
    ; If no match, then redirect to user code's USB_EP2 vector
    ljmp userUsbActivity


;-----------------------------------------------------------------------------
;  FUNCTION NAME:    EnterBootloader
;
;  DESCRIPTION:      
;    Prepares the processor then enters the bootloader - Entry
;    point for application.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:        None                  
;
;  RETURNS:          Does not return from this call
;
;  SIDE EFFECTS:     None    
;
;-----------------------------------------------------------------------------

_EnterBootloader::
    M8C_DisableGInt
    M8C_ClearWDT
    ; Load the RAM locations with the bootloader signature
    mov  [BootSignature], SIGNATURE0
    mov  [BootSignature+1], SIGNATURE1
    and  F, 0x00
    ljmp bootLoaderSoftBoot   ; restart


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootLoadVerify
;
;  DESCRIPTION:
;    Calculates checksum for user code Flash pages and then compares it with the
;    checksum stored in last Flash block.  If the checksum tallies
;    the control branches to __Start from where the user code can start 
;    normally.
;
;    If the checksum does not tally (previous download fail), the usercode 
;    function is called where critical operations can be performed and then 
;    jumps to bootloader code where, the processor waits for a 10 byte key from 
;    the host to initiate bootload operation
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootLoaderVerify::
    lcall bootCustomUserCode  ; execute custom user code first

bootLoaderSoftBoot::
    mov  [bootErrorCode], 0
  //  mov  [GbResetISR], 0

    ; If Project development phase, bypass bootloader
IF (DEBUG)
    ljmp __Start
ENDIF

    ; Check if the RAM locations have the bootloader mode
    ; signature.  If yes, then it means that the user code has written the 
    ; signature and wants to enter bootloader.  So bypass the Flash Verify 
    ; operation and directly jump to BootLoaderStart
    cmp  [BootSignature], SIGNATURE0
    jnz  DoFlashVerify
    cmp  [BootSignature+1], SIGNATURE1
    jnz  DoFlashVerify
    jmp  bootLoaderStart
    
DoFlashVerify:
    lcall bootVerifyFlashChecksum   ; Verify flash checksum
    tst   [bootErrorCode], IMAGE_VERIFY_ERROR    
    jnz   bootLoaderStart ; Verify failed, execute bootloader    
    ljmp  __Start   ; Flash checksum ok, start user code
    

;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootDelay
;
;  DESCRIPTION:
;    Delay a programmable number of sleep timer interrupts
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: Number of Sleep Timer interrupts to wait in A
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootDelay:
    mov  [bootTimeoutDelay], A
    and  reg[INT_CLR0], ~INT_MSK0_SLEEP   ; Clear any pending sleep interrupt

bootDelayLoop:
    ; Wait for sleep interrupt
    M8C_ClearWDT
    tst  reg[INT_CLR0], INT_MSK0_SLEEP
    jz   bootDelayLoop

    ; Decrement boot timer
    and  reg[INT_CLR0], ~INT_MSK0_SLEEP
    dec  [bootTimeoutDelay]
    jnz  bootDelayLoop
    ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootLoaderStart
;
;  DESCRIPTION:
;    Start of bootloader code for starting USB and downloading a new user
;    image.
;
;    The following operations are performed
;      1. Initialize the processor and start USB and wait till enumeration.
;      2. Wait for a 10 byte bootloader Key from host
;      3. If Key does not match loop to step 2
;      4. Receive Flash data from host and perform the requested tasks
;      5. Set the flags in bootErrorCode according to the status of operations
;      6. Wait for host to read the Error Code
;      7. Check if there was any error in the previous operation.  If yes reset.
;      8. Check if the BOOTLOAD_DONE flag is set.  If yes start the user code.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootLoaderStart:
    M8C_DisableGInt                
    M8C_ClearWDT
 //   mov  [GbResetISR], 0

    ; Clear the Signature before entering bootloader. So that any reset 
    ; generated in the bootloader will result in the normal process of 
    ; FlashVerify.  If this is not done here, even after a successful 
    ; bootloading operation and reset, the user code will never be entered.
    mov  [BootSignature], 0x00
    mov  [BootSignature+1], 0x00

    ; Setup bootloader stack
    mov   A, bootStackStart  ; Set stack to end of variables
    swap  SP, A

    ; Initialize Crystal Oscillator and PLL
    ; Set the CPU speed to 12MHz and the Sleep interval to 8Hz
    M8C_SetBank1
    mov   reg[OSC_CR0], (OSC_CR0_SLEEP_8Hz | OSC_CR0_CPU_12MHz)
    M8C_SetBank0

    ; Stop USB
    mov A, 8          ; number of sleep timer interrupts to delay
    call bootDelay
    lcall USB_Stop

    ; Disable all interrupts
    mov reg[INT_MSK3], 0x00 ; Write first to clear ENSWINT
    mov reg[INT_MSK2], 0x00
    mov reg[INT_MSK1], 0x00
    mov reg[INT_MSK0], 0x00

    ; Clear any pending interrupts which may
    ; have been set during the boot process.
    mov reg[INT_CLR2], 0x00 ; Clear any pending interrupts
    mov reg[INT_CLR1], 0x00
    mov reg[INT_CLR0], 0x00
    mov reg[INT_VC], 0             

    mov  [BootSignature], SIGNATURE0
    mov  [BootSignature+1], SIGNATURE1
   
    mov A, 2
    call bootDelay

    ; Enable Global Interrupts
    M8C_EnableGInt                

    ; Start USB and enumerate
    call InitializeUsb

    ; Reset the Error codes
    mov  [bootErrorCode], 0

    ; Wait for the Host to send the bootloader key
bootWaitForBootLoaderKey:
    mov  [bootTimeoutDelay], 0  ; disable timeout for read
    mov  A, 64                  ; expecting 10 byte buffer
    call ReadEp2
    
    ; Check if the first byte is 0xFF
    cmp [bootRamBuffer], 0ffh
    ; If yes, then proceed to check the command byte
    jz  CheckCommand
    ; If first byte is not 0xFF, then set the Invalid Key error bit
    or  [bootErrorCode], INVALID_KEY
    ; Write the Error code to IN endpoint
    jmp bootUpdateStatus
   
CheckCommand:
    ; Check if the command is Enter bootload command
    cmp  [bootRamBuffer+1], COMMAND_ENTER_BOOTLOAD
    ; If the command is Enter bootload, proceed to check the bootloader
    jz   CheckKey
    ; If command is not Enter Bootloader, set the Invalid command error
    or   [bootErrorCode], INVALID_COMMAND_ERROR
    ; Write the status to the IN endpoint
    jmp  bootUpdateStatus

CheckKey:
    ; Verify the 8 byte bootloader key
    call bootVerifyBootLoaderKey
    ; Check if the key is valid
    tst  [bootErrorCode], INVALID_KEY
    ; If not, update the IN endpoint with the error code and wait for
    ; the correct key
    jnz  bootUpdateStatus

    ; Clear the Comm checksum error flag
    and  [bootErrorCode], ~COMM_CHECKSUM_ERROR
    ; Start bootloader
    ljmp bootLoader
    
bootUpdateStatus:
    ; There was an error in the bootloader key.
    ; Write the Error Code to EP1
    call WriteErrorCodeToEP1

    ; Reset error code and loop Back to wait for the correct bootloader key
    mov  [bootErrorCode], 0
    ljmp bootWaitForBootLoaderKey                    


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootLoader
;
;  DESCRIPTION: The following is the sequence of operations performed.
;
;    1. Sets the BOOTLOAD_MODE flag to indicate bootload mode entered.
;    2. Wait for the host to read the status
;    3. Wait for 64 byte packet from host
;    4. Process the received packet
;    5. If any error encountered during processing, reset
;    6. If BOOTLOAD_DONE flag is set, execute user code
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  SIDE EFFECTS: NONE
;
;-----------------------------------------------------------------------------

bootLoader:
    and [bootErrorCode], ~IMAGE_VERIFY_ERROR
    or  [bootErrorCode], BOOTLOAD_MODE    ; Indicate BOOTLOAD mode entered
   
    ; Write the Error code to IN endpoint and wait for PC to read with timeout
    call WriteErrorCodeWait
   
bootReceiveNextFlashPage:
    ; Read 64 byte buffer from EP2 with a timeout
    mov [bootTimeoutDelay], BOOT_TIMEOUT
    mov  A, 64                            ; get a 64 byte buffer
    call ReadEp2
    
    ; Process the Received data packet
    call bootProcessFlashPage
 
    ; Update the IN Ep with error code and wait for PC to read with timeout
    call WriteErrorCodeWait

    ; The host has sent the packet and the packet has been processed.
    ; The host has read the error code.
    ; Now test the bootErrorcode for any error in the previous operation.
    ; If an error exists, perform a Software Reset.

    ; Test if the previous Flash Write failed because of
    ; flash protection
    tst  [bootErrorCode], FLASH_PROTECTION_ERROR
    jnz  ExecuteSwReset
    ; Test if the previous Flash Block Verify failed
    tst  [bootErrorCode], IMAGE_VERIFY_ERROR
    jnz  ExecuteSwReset
    ; Test if invalid key received
    tst  [bootErrorCode], INVALID_KEY
    jnz  ExecuteSwReset

    ; No Error in the previous operation.
    ; Test if BOOT_COMPLETED_OK flag is set
    tst  [bootErrorCode], BOOT_COMPLETED_OK    
    ; Loop till all the flash pages are done
    jz   bootReceiveNextFlashPage                

    ; Bootloading completed successfully.
    ; Execute a Software reset.  After this reset, the bootLoaderVerify will 
    ; be called to recheck the user code checksum and then the user code will 
    ; start executing.
    jmp  ExecuteSwReset


;-----------------------------------------------------------------------------
;  FUNCTION NAME: ExecuteSwReset
;
;  DESCRIPTION:
;    This function is called either if there is an error in the bootloading or 
;    if the bootload completed successfully.  So this function resets all the 
;    bootloader flags, stops the USB and executes a software reset.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

ExecuteSwReset:
    M8C_DisableGInt
    M8C_ClearWDTAndSleep
    M8C_SetBank0

    ; Disable and Clear all interrupts
    mov reg[INT_MSK3], 0x00 ; Write first to clear ENSWINT
    mov reg[INT_MSK2], 0x00
    mov reg[INT_MSK1], 0x00
    mov reg[INT_MSK0], 0x00

    mov reg[INT_CLR2], 0x00 ; Clear any pending interrupts
    mov reg[INT_CLR1], 0x00
    mov reg[INT_CLR0], 0x00

    ; Stop the USB
    lcall USB_Stop

    ; Clear memory
    mov X, 0                
clear_memory_loop:
    M8C_ClearWDT
    mov [X], 0
    inc X
    jnz clear_memory_loop

    ljmp bootLoaderSoftBoot  ; restart


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootLoaderTimeOut
;
;  DESCRIPTION:
;    
;    This function is called when a Read or Write timeout has occured because 
;    of long period of inactivity from host.  Execute a Reset
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootLoaderTimeOut:
    jmp  ExecuteSwReset

    
;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootProcessFlashPage
;
;  DESCRIPTION: 
;    The following is the sequence of operation
;    1. Verify if the bootLoader key is valid.  If not, skip processing 
;       received packet
;    2. If BOOTLOAD_DONE command received jmp to bootLoadDone part
;    3. Check if 1st half of flash data received.  If yes, store it in 
;       buffer and return
;    4. Check if 2nd half of flash data received.  If yes, store in buffer 
;       and go to step 5
;    5. Check if FLASH_WRITE command received.  If yes, write the 64 byte 
;       Flash data to the block indicated by blockId, and verify the write.
;    6. Check if FLASH_VERIFY command received. If yes, read the 64 bytes 
;       Flash data find checksum and compare with the recordChecksum received 
;       from the Master
;    7. If none of the above commands received, set the INVALID_COMMAND flag 
;       and exit
;-----------------------------------------------------------------------------
;  ARGUMENTS: 64 bytes received from the host
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootProcessFlashPage:
    M8C_ClearWDTAndSleep;

    ; Read the Block Id from the Ram Buffer
    mov [bootBlockId], [bootRamBuffer+10]
    mov [bootBlockId+1], [bootRamBuffer+11]
                                              
    ; Read the Command from the Ram Buffer
    mov [bootCommand], [bootRamBuffer+1]

    ; Check if Bootloader key is valid
    call bootVerifyBootLoaderKey
    tst [bootErrorCode], INVALID_KEY

    ; Exit if invalid bootloader key
    jnz bootSkipProcessFlashPage

    ; Check if BOOTLOAD_DONE command received   
    cmp [bootCommand], COMMAND_BOOTLOAD_DONE        
    jnz bootBypassBootLoadDone
       
    ; Bootload done command received
    ; Clear the BOOT_COMPLETED_OK flag    
    and [bootErrorCode], ~BOOT_COMPLETED_OK        

    ; Verify the Flash Checksum
    call bootVerifyFlashChecksum                

    ; Check if Flash checksum was successful
    tst [bootErrorCode], IMAGE_VERIFY_ERROR        
    jnz bootSkipProcessFlashPage

    ; If Flash checksum ok, set the boot completed ok flag and exit
    or [bootErrorCode], BOOT_COMPLETED_OK
bootSkipProcessFlashPage:
    ret

bootBypassBootLoadDone:
    ; Verify Checksum of the received packet
    call bootVerifyPacketChecksum

    ; Exit if there is packet checksum error
    tst [bootErrorCode], COMM_CHECKSUM_ERROR
    jnz bootSkipProcessFlashPage

    ; Test if the Received Flash page is First or second half of flash page
    tst [bootRamBuffer+12], 0x01               
    jnz ProcessSecondHalf

    ; First half of Flash Page received.  
    ; Call the StoreFlashData function with an offset of 0
    mov  A, 0
    call StoreFlashData

    jmp  ReturnProcessFlashPage
    
ProcessSecondHalf:
    ; Second half of Flash page received
    ; Call the StoreFlashData function with an offset of 32
    mov  A, 32
    call StoreFlashData
    
    ; Check if FLASH_WRITE command received
    cmp [bootCommand], COMMAND_FLASH_WRITE        
    jnz bootTestFlashVerify
    
    ; Write Buffer to Flash and verify
    call bootFlashWrite
    call bootVerifyFlashBlock
ReturnProcessFlashPage:
    ret

bootTestFlashVerify:
    ; Check if Flash Verify command has been received
    cmp [bootCommand], COMMAND_FLASH_VERIFY

    ; If Flash Verify Command not received, then, set invalid command error
    jnz bootInvalidCommand

    ; If Flash Verify command received, read the Flash block and compare it 
    ; with the received Flash data
    call bootVerifyFlashBlock
    ret

bootInvalidCommand:
     ; If no recognised command received, set the INVALID_COMMAND flag
     or [bootErrorCode], INVALID_COMMAND_ERROR    
     ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: StoreFlashData
;
;  DESCRIPTION: 
;    This function transfers the 32 byte flash data from bootRamBuffer to 
;    bootFlashBuffer
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: 
;    A contains the offset where the data is to be stored.  For storing the 
;    first half of flash data, A contains 0.  For storing the 2nd half of flash 
;    data A contains 32
;
;-----------------------------------------------------------------------------

StoreFlashData:
    ; Set the Source pointer to bootRamBuffer+13
    mov  [bootSourcePointer], <bootRamBuffer+13

    ; Set the Destination pointer to bootFlashBuffer
    mov  [bootDestPointer], <bootFlashBuffer

    ; Add the offset to LSB of destination pointer
    add  [bootDestPointer], A

    ; Init byte count to 32
    mov  [bootByteCount], 32

ReadNextByte2:
    M8C_ClearWDT
    ; Read from Source
    mvi  A, [bootSourcePointer]

    ; Write to Destination
    mvi  [bootDestPointer], A

    ; Loop till all bytes transferred
    dec  [bootByteCount]
    jnz  ReadNextByte2
    ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootVerifyFlashBlock
;
;  DESCRIPTION: 
;    This function reads 64 bytes from Flash block indicated by blockId and 
;    and compares the read data with the data in bootFlashBuffer. If the 
;    data are same, the FLASH_CHECKSUM_ERROR flag is cleared.  Otherwise it 
;    is set.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: bootBlockID - FLASH block ID to compare
;             bootFlashBuffer - buffer to compare FLASH block against
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootVerifyFlashBlock:
    ; Set FLASH_CHECKSUM_ERROR bit, default to error
    or   [bootErrorCode], FLASH_CHECKSUM_ERROR

    ; Initialize variables
    mov  [bootByteCount], 64
    mov  [bootRomAddress], [bootBlockId+1]
    mov  [bootRomAddress+1], 0

    ; compute block address
    and  F, ~FLAG_CARRY
    rrc  [bootRomAddress]
    rrc  [bootRomAddress+1]
    rrc  [bootRomAddress]
    rrc  [bootRomAddress+1]

    mov  A, 0
    mov  X, A

    ; Read from the Current address pointed by bootRomAddress and compare
bootVerifyLoop:
    push X
      mov  A, [bootRomAddress]
      mov  X, [bootRomAddress+1]
      romx
    pop  X

    cmp  A, [X+bootFlashBuffer]
    jnz  FlashVerifyError

    ; next byte
    inc  X
    inc  [bootRomAddress+1]
    dec  [bootByteCount]
    jnz  bootVerifyLoop

FlashVerifyOk:
    ; Clear FLASH_CHECKSUM_ERROR for valid verify
    and [bootErrorCode], ~FLASH_CHECKSUM_ERROR     

FlashVerifyError:
    ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootVerifyBootLoaderKey
;
;  DESCRIPTION:
;    Compares the 8 byte key, KEY1 to KEY8 received from host with the values 
;    declared in the bootloader.inc file.  If the keys match the INVALID_KEY 
;    flag is cleared.  If the keys do not match, the INVALID_KEY flag is set.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  SIDE EFFECTS: NONE
;
;-----------------------------------------------------------------------------

bootVerifyBootLoaderKey:
    and [bootErrorCode], ~INVALID_KEY
    cmp [bootRamBuffer+2], KEY1
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+3], KEY2
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+4], KEY3
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+5], KEY4
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+6], KEY5
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+7], KEY6
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+8], KEY7
    jnz bootInvalidBootLoaderKey
    cmp [bootRamBuffer+9], KEY8
    jnz bootInvalidBootLoaderKey
    ret

bootInvalidBootLoaderKey:
    or [bootErrorCode], INVALID_KEY
    ret
     

;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootVerifyPacketChecksum
;
;  DESCRIPTION: 
;    Verifies the checksum of the received 64 byte data packet.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: 64 bytes received from host stored in bootRamBuffer
;
;  RETURNS: Clears the COMM_CHECKSUM_ERROR bit in the bootErrorCode register 
;            if checksum is ok.  Sets the bit if checksum failed
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootVerifyPacketChecksum:
    or [bootErrorCode], COMM_CHECKSUM_ERROR

    ; Initialize pointer to start of bootRamBuffer
    mov X, bootRamBuffer

    ; Checksum to be calculated for 45 bytes    
    mov [bootByteCount], 45

    ; Clear the Accumulator. Accumulator will hold the calculated checksum
    mov A, 0                        

bootAddNextByteOfPacket:
    ; Add the byte pointed by X to Accumulator 
    add A, [X]

    ; Increment the Index
    inc X

    ; Check if the checksum has been calculated for all the bytes
    dec [bootByteCount]
    jnz bootAddNextByteOfPacket

    ; Now X points to the packet checksum in the buffer
    ; Compare the received packet checksum with the calculated one
    cmp A, [X]                    
    ; Branch to Checksum Error if these values do not match
    jnz bootCheckSumError

bootChecksumOk:
    ; We are here if the checksum of the received packet is ok.
    ; So clear the CHECKSUM_ERROR Flag
    and [bootErrorCode], ~COMM_CHECKSUM_ERROR

bootCheckSumError:
    ; Restore the page mode to native paging
    ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootVerifyFlashChecksum
;
;  DESCRIPTION:
;    Calculates the user Image checksum  and compares with the checksum 
;    stored in the last Flash block.  If checksum ok, IMAGE_VERIFY_ERROR
;    bit is cleared in the bootErrorCode.  Otherwise set
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: NONE
;
;  RETURNS: Clears the IMAGE_VERIFY_ERROR bit in the bootErrorCode register 
;           if checksum is ok.  Sets the bit if checksum failed
;
;  SIDE EFFECTS: NONE
;-----------------------------------------------------------------------------

bootVerifyFlashChecksum:
    ; Clear the IMAGE_VERIFY_ERROR flag
    and [bootErrorCode], ~IMAGE_VERIFY_ERROR

    ; Initialize the bootRomAddress to Starting Address of
    ; User code
    mov [bootRomAddress], >START_ADDRESS
    mov [bootRomAddress+1], <START_ADDRESS

    ; Initialize the Number of Blocks for user code
    mov [bootBlockCount], >NUMBER_OF_BLOCKS
    mov [bootBlockCount+1], <NUMBER_OF_BLOCKS
     
    ; Initialize the Flash Checksum Variable to zero
    mov [bootFlashChecksum], 0
    mov [bootFlashChecksum+1], 0

    ; Block Loop
bootChecksumNextPage:
    ; Initialize Byte so that 64 bytes from each block
    ; have to be added to checksum
    mov [bootByteCount], 64

bootChecksumNextByte:
    ; Clear Watchdog
    M8C_ClearWDT

    ; Read from the Current address pointed by bootRomAddress
    mov A, [bootRomAddress]
    mov X, [bootRomAddress+1]
    romx

    ; Add this byte to Checksum
    add [bootFlashChecksum+1], A
    adc [bootFlashChecksum], 0

    ; Increment the Address variable
    add [bootRomAddress+1], 1
    adc [bootRomAddress], 0
     
    ; Check if 64 bytes have been added
    dec [bootByteCount]

    ; If not get the next byte
    jnz bootChecksumNextByte
     
    ; Decrement the Block Count
    sub [bootBlockCount+1], 1            ; decrement block count
    jnz bootChecksumNextPage
    sbb [bootBlockCount], 0

    ; If not zero, go to next block
    jnz bootChecksumNextPage
     
    ; All the memory locations in the user code have been added
    ; Now Read the checksum high byte
    mov X, <CHECKSUM_ADDR_H
    mov A, >CHECKSUM_ADDR_H
    romx

    ; Compare this with MSB of Calculated checksum
    cmp A, [bootFlashChecksum]

    ; Checksum Fail, if the values do not match
    jnz bootFlashCheckSumFail
    
    ; Read from checksum low byte
    mov X, <CHECKSUM_ADDR_L
    mov A, >CHECKSUM_ADDR_L
    romx

    ; Compare with LSB of calculated Checksum
    cmp A, [bootFlashChecksum+1]            

    ; Checksum Fail, if the values do not match
    jnz bootFlashCheckSumFail
    ret

bootFlashCheckSumFail:
    ; Set the IMAGE_VERIFY_ERROR if Flash checksum did not match
    or [bootErrorCode], IMAGE_VERIFY_ERROR
    ret


;-----------------------------------------------------------------------------
;  FUNCTION NAME: bootFlashWR
;
;  DESCRIPTION:
;    Writes the Flash data to flash block indicated by bootBlockId
;----------------------------------------------------------------------------
;
;  ARGUMENTS: 
;    bootBlockId - block to be written
;
;  RETURNS:  NONE
;            
;  SIDE EFFECTS: NONE
; 
;  NOTE:
;  If your are using ICE, change flashsecurity block to 'U'
;  If your are using silicon, change flashsecurity block to 'R'
;-----------------------------------------------------------------------------

bootFlashWrite:
    ; Disable Global Interrupts
    M8C_DisableGInt    

    ; Clear the FLASH_PROTECTION_ERROR Flag
    and  [bootErrorCode], ~FLASH_PROTECTION_ERROR    ; clear memory error flag
    
    ; Push the Block number to stack
    mov  A, [bootBlockId+1]
    push A                            

    ; Push source pointer to stack
    mov  A, <bootFlashBuffer
    push A                            

    ; Write Block to Flash
    lcall FlashWriteLite                    

    ; Restore Stack    
    add  SP, -2                        

    ; Check if Flash Write was success. 0 in A means success
    cmp  A, 0
    jz   FlashWriteSuccess

    ; If Flash write failed, set the FLASH_PROTECTION_ERROR flag
    or   [bootErrorCode], FLASH_PROTECTION_ERROR

FlashWriteSuccess:
    ; Enable Global Interrupts
    M8C_EnableGInt
    ret
     

;------------------------------------------------------------------------------
;    FUNCTION NAME: ReadEp2
;
;    DESCRIPTION:
;      Transfers A bytes from EP2 to the bootRamBuffer
;
;------------------------------------------------------------------------------
;
;    ARGUMENTS: A contains number of bytes to read from EP2
;                [bootTimeoutDelay]: 0 = disabled, n = delay in sleep ticks
;                            
;    RETURNS:
;      Data read is returned in the bootRamBuffer.
;
;    SIDE EFFECTS: A and X registers are modified
;
;------------------------------------------------------------------------------

ReadEp2:
    ; Save number of bytes to read from EP2
    mov   [bootByteCount], A
    mov   [bootDestPointer], <bootRamBuffer
    and   reg[INT_CLR0], ~INT_MSK0_SLEEP   ; Clear pending sleep interrupts

ep2NextPacket:
    ; Enable EP2 to receive the Bootloader packets
    mov   [bEp2Count], 0
    mov   A, 2 
    lcall USB_EnableOutEP

ep2WaitData:
    M8C_ClearWDT
    //mov   A, [GbResetISR]       ; check for USB reset
    //jnz   bootLoaderStart       ; reset USB and restart the bootloader if set
    mov   A, [bEp2Count]        ; check for data received
    jnz   ep2DataAvail
    mov   A, [bootTimeoutDelay]
    jz    ep2WaitData           ; delay is disabled if == 0
    tst   reg[INT_CLR0], INT_MSK0_SLEEP
    jz    ep2WaitData
    and   reg[INT_CLR0], ~INT_MSK0_SLEEP
    dec   [bootTimeoutDelay]
    jnz   ep2WaitData
    jmp   bootLoaderTimeOut

ep2DataAvail:
    mov  A, 2
    lcall USB_bGetEPCount       ; get number of bytes received
    mov  [bootTmp], A
    cmp  A, [bootByteCount]     ; don't read more than buffer size
    jc   ep2LoopInit
    mov  [bootTmp], [bootByteCount]

ep2LoopInit:
    mov  X, 0

nextEp2Loop:    
    M8C_ClearWDT
    mov  A, REG[X+EP2DATA]      ; read USB packet into buffer
    mvi  [bootDestPointer], A
    inc  X
    dec  [bootTmp]
    jnz  nextEp2Loop
    mov  A, X
    sub  [bootByteCount], A     ; reduce buffer count
    jnz  ep2NextPacket

    ret   


;------------------------------------------------------------------------------
;    FUNCTION NAME: WriteErrorCodeToEP1
;
;    DESCRIPTION:
;      Transfers the bootErrorCode to the EP1 buffer
;
;------------------------------------------------------------------------------
;
;    ARGUMENTS: NONE
;                            
;    RETURNS: NONE
;
;    SIDE EFFECTS: A and X registers are modified
;
;------------------------------------------------------------------------------

WriteErrorCodeToEP1:
    mov  [bEp1Count], 0
    mov  [USB_APIEPNumber], 1   ; Endpoint number 1
    mov  [USB_APICount], 1      ; EP Count = 1
    mov  A, <bootErrorCode      ; Address of buffer to send
    mov  X, A
    lcall USB_XLoadEP          ; Call XLoadEP function to load EP1
    ret


;------------------------------------------------------------------------------
;    FUNCTION NAME: WaitEp1Read
;
;    DESCRIPTION:
;      Wait for the PC to read the EP1 status
;
;------------------------------------------------------------------------------
;
;    ARGUMENTS: NONE
;                            
;    RETURNS: NONE
;
;    SIDE EFFECTS: A and X registers are modified
;
;------------------------------------------------------------------------------

WriteErrorCodeWait:
    call  WriteErrorCodeToEP1

    ; Now initialize the timeout
    mov   [bootTimeoutDelay], BOOT_TIMEOUT
    ; Clear any pending sleep interrupt
    and   reg[INT_CLR0], ~INT_MSK0_SLEEP

bootWaitForMasterToReadStatus1:
    M8C_ClearWDT
    cmp   [bootTimeoutDelay], 0   ; boot delay disabled
    jz    CheckEp1Count1
    ; Check if the Sleep bit is set in INT_CLR0 register
    tst   reg[INT_CLR0], INT_MSK0_SLEEP
    ; If not set, go and check if Host has read from IN endpoint
    jz    CheckEp1Count1

    ; A sleep interrupt has occured.  So clear the bit in INT_CLR0 retister
    and   reg[INT_CLR0], ~INT_MSK0_SLEEP
    ; Decrement timeout variable
    dec   [bootTimeoutDelay]
    ; If timeout variable becomes zero, then a timeout has occured
    jz    bootLoaderTimeOut

CheckEp1Count1:
    ; Call the function to check if Host has read EP1
    mov   A, [bEp1Count]
    ; If not wait for Host to read status or timeout
    jz    bootWaitForMasterToReadStatus1

    ret


;------------------------------------------------------------------------------
;    FUNCTION NAME: InitializeUsb
;
;    DESCRIPTION:
;      Starts USB and waits for the host to enumerate.
;
;------------------------------------------------------------------------------
;
;    ARGUMENTS: None
;                            
;    RETURNS: NONE
;
;    SIDE EFFECTS: A and X registers are modified
;
;------------------------------------------------------------------------------

InitializeUsb:
    ; Start USB
    mov   A, 0xFF
    lcall USB_Start

    ; Wait for the USB to enumerate.
WaitForEnumeration:
    M8C_ClearWDT
    lcall USB_bGetConfiguration
    jz    WaitForEnumeration

    ; Reset ISR variables
    mov   [bEp1Count], 0
    mov   [bEp2Count], 0
    //mov   [GbResetISR], 0

    ; Enable EP interrupts
    or    reg[INT_MSK1], (INT_MSK1_USB_EP1 | INT_MSK1_USB_EP2)

    ret
    
;------------------------------------------------------------------------------
;  End of File
;------------------------------------------------------------------------------


